/* mbed Microcontroller Library
 * Copyright (c) 2006-2013 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "mbed.h"
#include "BLEDevice.h"
#include "UARTService.h"
#include "ble_gap.h"
#include "Gap.h"

 #define NEED_CONSOLE_OUTPUT 1 /*  Set this if you need debug messages on the console;
                               * it will have an impact on code-size and power consumption. */

 #if NEED_CONSOLE_OUTPUT
#define DEBUG(...) { printf(__VA_ARGS__); }
#else
#define DEBUG(...)  
#endif  #if NEED_CONSOLE_OUTPUT 

#define LED2OFF "led2 off"
#define LED2ON "led2 on"
int initialLEDSTATE = 0;        /* added for alternate testing */  
int TSTLEDSTATE = 0;           /* added for alternate testing */

char buff[100];
//char rxPayload[CMD_SIZE];

BLEDevice  ble;                               // Create Bluetooth object
DigitalOut led1(LED1);                        // Set the pin attached to LED1 as an output
DigitalOut led2(p19);                        // Set the pin attached to LED2 as an output
uint16_t currhandle;
UARTService *uartServicePtr;

/*  Start RSSI sensing algorthym ADDED for alternate testing*/
//void RSSIst()
//{
//uint32_t sd_ble_gap_rssi_start (uint16_t conn_handle, uint8_t threshold_dbm, uint8_t  skip_count);  /* start RSSI sensing */
  //      {  
  //uint32_t sd_ble_gap_rssi_get (uint16_t  conn_handle, int8_t *  p_rssi );  /* get RSSI signal strength event */
    //if 
     //   (conn_handle = NULL) & (TSTLEDSTATE = 1);
       // return;    /*drop out of fuction */
    //else   
      //  for (p_rssi != null)&(TSTLEDSTATE = 0);               /* if RSSI signal is not null and LED state 0 spin loop*/
        //    led2 = 1, TSTLEDSTATE = 1;                        /* set led2 = 1 turn on*/
          //  if (p_rssi = null) & initialLEDSTATE = 0;         /* test if RSSI signal is null yet if null set LED to 0 or off */
            //    led2 = 0, TSTLEDSTATE = 0;                    /* set led 2 to off and set test led back to 0*/
            //}
//}
//const ble_gap_evt_t * const p_gap_evt = &p_ble_evt->evt.gap_evt;
uint16_t conss;
uint32_t error1;
/* BLE disconnected callback */
void disconnectionCallback(Gap::Handle_t handle, Gap::DisconnectionReason_t reason)
{
    sd_ble_gap_rssi_stop( handle);
    sprintf(buff," 0x%08X\n",handle);
    DEBUG(buff);
    DEBUG("Disconnected!\n\r");
    DEBUG("Restarting the advertising process\n\r");
    ble.startAdvertising();
}
int8_t p_rssi;
/* BLE UART data received callback */
void onDataWritten(const GattCharacteristicWriteCBParams *params)
{
    if ((uartServicePtr != NULL) && (params->charHandle == uartServicePtr->getTXCharacteristicHandle())) {     //If characters received over BLE
        uint16_t bytesRead = params->len;
        DEBUG("received %u bytes\n\r", bytesRead);
        DEBUG("Received string: '");
        DEBUG((const char *)params->data);             //Note the size of data expands to the largest string received. Need to use bytesRead to resize.
        DEBUG("'\n\r");
        if (!strncmp(LED2ON,(const char *)params->data,bytesRead))  led2 = 1;   // If the received and command string are equal turn led2 on. Note strcmp returns 0 if strings are equal
                                                    TSTLEDSTATE = 1;
        if (!strncmp(LED2OFF,(const char *)params->data,bytesRead)) led2 = 0;   // If the received and command string are equal turn led2 off.
                                                  TSTLEDSTATE = 0;
          //if(p_gap_evt->conn_handle) ;                                                                             //Toggle LED2 when data received
        ble.updateCharacteristicValue(uartServicePtr->getRXCharacteristicHandle(), params->data, bytesRead);   // Echo received characters back over BLE
    
    }
}

void onConnectionCallback( Gap::Handle_t handle, Gap::addr_type_t type, const Gap::address_t ownAddr,
                            const Gap::ConnectionParams_t *params ){

    DEBUG( "Connected!!\r\n" );
    error1=sd_ble_gap_rssi_start(handle);
    sprintf(buff," 0x%08X\n",error1);
    currhandle=handle;
    DEBUG(buff);

}

/* Periodic Ticker callback */
void periodicCallback(void)
{
    //led1 = !led1;                             // Toggle LED 1
    if(ble.getGapState().connected)
    {
    //led1=1;
    led2=1;
    }
    else
    {
        led2=0;
        //led2=1;
    }
    
   
}

int main(void)
{
    led1 = 1;
    led2 = 0;
    Ticker ticker;                            // Create period timer
    ticker.attach(periodicCallback, 1);       // Attach ticker callback function with a period of 1 second

    DEBUG("Initialising the nRF51822\n\r");
    ble.init();
    ble.onConnection( onConnectionCallback );
    ble.onDisconnection(disconnectionCallback);                                            // Define callback function for BLE disconnection event
    ble.onDataWritten(onDataWritten);                                                      // Define callback function for BLE Data received event

    /* setup advertising */
    ble.accumulateAdvertisingPayload(GapAdvertisingData::BREDR_NOT_SUPPORTED| GapAdvertisingData::LE_GENERAL_DISCOVERABLE);             // Indicate that Legacy Bluetooth in not supported
    ble.setAdvertisingType(GapAdvertisingParams::ADV_CONNECTABLE_UNDIRECTED);
    ble.accumulateAdvertisingPayload(GapAdvertisingData::SHORTENED_LOCAL_NAME,
                                     (const uint8_t *)"GEN 10", sizeof("GEN 10") - 1);
    ble.accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LIST_128BIT_SERVICE_IDS,
                                     (const uint8_t *)UARTServiceUUID_reversed, sizeof(UARTServiceUUID_reversed));
                                     
 /* while  (UARTServiceUUID_reversed != NULL){    //  test code for first event UARTservice to be REMOVED
        led2 = 1;
        wait(1000);
        led2 = 0;
} */
    ble.setAdvertisingInterval(Gap::MSEC_TO_ADVERTISEMENT_DURATION_UNITS(1000));          // Set advertising interval to 1 second
    ble.startAdvertising();                                                               // Start advertising


    UARTService uartService(ble);                                                         // Create BLE UART service
    uartServicePtr = &uartService;                                                        // Initalise pointer to point to UART Service


    while (true) {
        
        ble.waitForEvent();                                                               // Wait for BLE events
    }
}

